<?php
/*
Plugin Name: PW_Archives
Plugin URI: http://philipwalton.com/2011/02/08/pw_archives/
Description: An extremely light-weight, easy to use, and fully-customizable archiving plugin that allows you to display your posts almost any way you like.
Version: 1.0.3
Author: Philip Walton
Author URI: http://philipwalton.com
*/

/* 
 * Call this function from your theme and pass arguments as desired.
 */
function PW_Archives($args = array()) {
	$pwa = new PW_Archives($args);
	$pwa->display();
}


/* 
 * PW_Form helper class
 */
class PW_Form {
			
	public static function encode($input) {
		return htmlspecialchars($input, ENT_QUOTES, 'UTF-8');
	}

	public static function getIdFromName($name)
	{
		return str_replace(array('[]', '][', '[', ']'), array('', '_', '_', ''), $name);
	}
	
	public static function tag($name, $htmlOptions=array(), $content=null) {
		$output = "<$name";
		foreach($htmlOptions as $property=>$value) {
			$output .= " {$property}=\"{$value}\"";
		}
		$output .= ($content !== null) ? ">$content</$name>" : " />";
		return $output;
	}

	public static function select($name, $items, $selected='', $htmlOptions=array()) {
		$htmlOptions['name'] = $name;
		$htmlOptions['id'] = self::getIdFromName($name);
		$content = '';
		foreach($items as $value=>$option) {
			$atts = array('value'=>$value);
			if ($selected === (string) $value)
				$atts['selected'] = 'selected';			
			$content .= self::tag('option', $atts, $option);
		}
		return self::tag('select', $htmlOptions, $content);
	}

	public static function radioButtonList($name, $items, $selected, $htmlOptions=array()) {
		$htmlOptions['name'] = $name;
		$htmlOptions['type'] = 'radio';
		foreach($items as $value=>$option) {
			$htmlOptions['value'] = $value;
			if ($selected === (string) $value) {				
				$htmlOptions['checked'] = 'checked';
			} else if (isset($htmlOptions['checked'])) {
				unset($htmlOptions['checked']);
			}
			$temp_items[] = self::tag('label', array(), self::tag('input', $htmlOptions, " $option"));
		}
		return implode($temp_items, isset($htmlOptions['separator']) ? $htmlOptions['separator'] : "");
	}
	
	public static function checkbox($name, $item, $selected, $htmlOptions=array()) {
		$value = array_keys($item); $value = $value[0];
		$label = array_values($item); $label = $label[0];
		
		$htmlOptions['name'] = $name;
		$htmlOptions['type'] = 'checkbox';
		$htmlOptions['value'] = $value;
		if ($selected)
			$htmlOptions['checked'] = 'checked';
		else if (isset($htmlOptions['checked']))
			unset($htmlOptions['checked']);
		$output = isset($htmlOptions['uncheckValue']) ? self::tag('input', array('type'=>'hidden', 'value'=>$htmlOptions['uncheckValue'], 'name'=>$name)) : "";
		$output .= self::tag('label', array(), self::tag('input', $htmlOptions, " $label"));
		return $output;
	}

	public static function textfield($name, $value, $htmlOptions = array()) {
		$htmlOptions['type'] = 'text';
		$htmlOptions['value'] = self::encode($value);
		$htmlOptions['name'] = $name;
		$htmlOptions['id'] = self::getIdFromName($name);
		return self::tag('input', $htmlOptions);
	}
} // Class PW_Form

class PW_Archives_Form extends PW_Form {

	protected $options; // the options name to query from the wp_options table
	
	public function __construct($options = array()) {
		if ($options) {
			$this->options = $options;
		}
	}

	public function general_options() {
		// do nothing
	}
	public function year_options() {
		// do nothing
	}
	public function month_options() {
		// do nothing
	}
	public function post_options() {
		// do nothing
	}
	public function order() {
		$items = array("DESC"=>"Descending", "ASC"=>"Ascending");
		echo self::select('PW_Archives_options[order]', $items, $this->options['order']);
	}
	
	public function year_count() {
		$items = array(
			"ALL"=>"All",
			"1"=>"1",
			"2"=>"2",
			"3"=>"3",
			"4"=>"4",
			"5"=>"5",
			"6"=>"6",
			"7"=>"7",
			"8"=>"8",
			"9"=>"9",
		);
		echo self::select('PW_Archives_options[year_count]', $items, $this->options['year_count']);
		echo ' <span class="description">(e.g. selecting \'3\' would display only the 3 most recent years)</span>';
	}

	public function year_format() {
		echo self::textfield('PW_Archives_options[year_format]', $this->options['year_format'], array('class'=>'small-text code'));
	 	echo ' <span class="description">See the <a href="http://php.net/manual/en/function.date.php">PHP date() documentation</a> for options</span>';
	}
	public function year_template() {
		echo self::textfield('PW_Archives_options[year_template]', $this->options['year_template'], array('class'=>'large-text code'));
		echo '<small>Variables: <code>%YEAR%</code>, <code>%YEAR_URL%</code>, and <code>%POST_COUNT%</code></small>';
	}
	public function month_expand() {
		$items = array(
			"ALL"=>"Display month links for all visible years",
			"1"=>"Only display month links in the most recent year",
			"0"=>"Don't display links for individual months",
		);
		echo self::radioButtonList('PW_Archives_options[month_expand]', $items, $this->options['month_expand'], array('separator'=>'<br />'));
	}
	public function month_format() {
		echo self::textfield('PW_Archives_options[month_format]', $this->options['month_format'], array('class'=>'small-text code'));
		echo ' <span class="description">See the <a href="http://php.net/manual/en/function.date.php">PHP date() documentation</a> for options</span>';
	}
	public function month_template() {
		echo self::textfield('PW_Archives_options[month_template]', $this->options['month_template'], array('class'=>'large-text code'));
		echo '<small>Variables: <code>%MONTH%</code>, <code>%MONTH_URL%</code>, and <code>%POST_COUNT%</code></small>';
	}
	public function show_posts() {
		echo self::checkbox('PW_Archives_options[show_posts]', array('YES'=>'Display post links within visible months'), $this->options['show_posts'] == "YES", array('uncheckValue'=>'NO'));
	}
	public function faster_permalinks() {
		echo self::checkbox('PW_Archives_options[faster_permalinks]', array('YES'=>'Use PW_Archive\'s faster permalinks option if possible'), $this->options['faster_permalinks'] == "YES", array('uncheckValue'=>'NO'));
		echo '<br /><small>IMPORTANT: Faster Permalinks is an experimental feature. Do not use if you have a very customized permalink structure.</small>';
	}
	public function post_date_format() {
		echo self::textfield('PW_Archives_options[post_date_format]', $this->options['post_date_format'], array('class'=>'small-text code'));
		echo ' <span class="description">See the <a href="http://php.net/manual/en/function.date.php">PHP date() documentation</a> for options</span>';
	}
	public function post_template() {
		echo self::textfield('PW_Archives_options[post_template]', $this->options['post_template'], array('class'=>'large-text code'));
		echo '<small>Variables: <code>%POST_TITLE%</code>, <code>%POST_DATE%</code>, <code>%POST_URL%</code>, and <code>%COMMENT_COUNT%</code></small>';
	}
	public function validate($input) {
		$input = wp_parse_args($input, PW_Archives::$defaults);
		return $input; // return validated input
	}
} // class PW_Archives_Form	

class PW_Archives {
	
	public static $defaults = array(	
		'title' => 'Archives', // the default title used in a PW_Archives widget, ignored in non-widget use	
		'order' => 'DESC',
		
		'year_count' => 'ALL', // values: int, or all
		'year_format' => 'Y',
		'year_template' => '<a href="%YEAR_URL%">%YEAR%</a> (%POST_COUNT%)',
		
		'month_expand' => 'ALL', // values: int, or all
		'month_format' => 'F',
		'month_template' => '<a href="%MONTH_URL%">%MONTH%</a> (%POST_COUNT%)',
		
		'show_posts' => 'YES',
		'faster_permalinks' => 'YES',
		'post_date_format' => 'jS',
		'post_template' => '%POST_DATE%: <a href="%POST_URL%">%POST_TITLE%</a> (%COMMENT_COUNT%)',
	);
	protected $posts = array();
	protected $options;		

	public static function get_options() {
		$settings = get_option('PW_Archives_options');
		if ($settings) {
			$settings = wp_parse_args($settings, self::$defaults);
		} else {
			add_option('PW_Archives_options', self::$defaults, '', 'no');
			$settings = self::$defaults;
		}
		return $settings;
	}

	public static function initialize_settings() {
		
		$pwaf = new PW_Archives_Form(self::get_options());
		
		// Register a setting and its sanitization callback.
		register_setting('PW_Archives_options', 'PW_Archives_options', array($pwaf, 'validate') );
		
		add_settings_section('general_options', 'General Options', array($pwaf, 'general_options'), __FILE__);
			add_settings_field('order', 'Display order:', array($pwaf, 'order'), __FILE__, 'general_options');
		add_settings_section('year_options', 'Year Options', array($pwaf, 'year_options'), __FILE__);
			add_settings_field('year_count', 'Number of years to display:', array($pwaf, 'year_count'), __FILE__, 'year_options');
			add_settings_field('year_format', 'Date Format:', array($pwaf, 'year_format'), __FILE__, 'year_options');
			add_settings_field('year_template', 'Template:', array($pwaf, 'year_template'), __FILE__, 'year_options');
		add_settings_section('month_options', 'Month Options', array($pwaf, 'month_options'), __FILE__);
			add_settings_field('year_count', 'Layout:', array($pwaf, 'month_expand'), __FILE__, 'month_options');
			add_settings_field('month_format', 'Date Format:', array($pwaf, 'month_format'), __FILE__, 'month_options');
			add_settings_field('month_template', 'Template:', array($pwaf, 'month_template'), __FILE__, 'month_options');
		add_settings_section('post_options', 'Post Options', array($pwaf, 'post_options'), __FILE__);
			add_settings_field('show_posts', 'Layout:', array($pwaf, 'show_posts'), __FILE__, 'post_options');
			add_settings_field('faster_permalinks', 'Faster Permalinks:', array($pwaf, 'faster_permalinks'), __FILE__, 'post_options');
			add_settings_field('post_date_format', 'Post Date:', array($pwaf, 'post_date_format'), __FILE__, 'post_options');
			add_settings_field('post_template', 'Template:', array($pwaf, 'post_template'), __FILE__, 'post_options');
	}
	
	public static function settings_form() {
		?><div class="wrap">
			<?php screen_icon("options-general"); ?>
			<h2>PW_Archives</h2>
			<p>These are the default settings for all uses of PW_Archives in your theme -- as a widget, as shortcode, or hard-coded into your php files. These defaults can always be overridden for individual instances use. This can be particularly helpful if you're using PW_Archives in more than one place on your site (e.g. an abbreviated list of archives on your sidebar as well as a complete list on a sitemap page).</p>
			<form action="options.php" method="post">
				<?php settings_fields('PW_Archives_options'); ?>
				<?php do_settings_sections(__FILE__); ?>
				<p class="submit">
					<input name="Submit" type="submit" class="button-primary" value="<?php esc_attr_e('Save Changes'); ?>" />
					
				</p>
				
			</form>
		</div><?php
	}
	
	public static function register_widget() {
		register_widget('PW_Archives_Widget');
	}
	
	// Adds the shortcode [PW_Archives]. Called via add_shortcode(). 
	public static function shortcode( $atts = array() ) {
		$options = get_option('PW_Archives_options');	
		if (!$options) {
			add_option('PW_Archives_options', PW_Archives::$defaults, '', 'no');
			$options = PW_Archives::$defaults;
		}
		$atts = ($atts) ? shortcode_atts( $options, $atts ) : $options;
		$pwa = new PW_Archives($atts);
		return $pwa->display(false);
	}
	
	// Adds a submenu item to the "Settings" menu. Called from the 'admin_menu' action.
	public static function add_submenu() {
		add_submenu_page( 'options-general.php', 'PW_Archives', 'PW_Archives', 'manage_options', __FILE__, array('PW_Archives', 'settings_form'));
	}
	
	// Deletes the option from the wp_options table. Called from add_action('deactivate_[...]')
	public static function delete_option() {
		delete_option('PW_Archives_options');
	}
	
	// Add settings link on plugin page. Called from add_filter('plugin_action_links_[...]')
	public static function settings_link($links) {
	  $settings_link = '<a href="options-general.php?page='.plugin_basename(__FILE__).'">Settings</a>'; 
	  array_unshift($links, $settings_link); 
	  return $links; 
	}
		
	// Constructor
	public function PW_Archives( $args = array() ) {
		// Use passed $args if it's provided
		if ($args) {
			$this->options = wp_parse_args($args, self::$defaults);
		} else {
			// Get the saved options from the database
			$this->options = self::get_options();
		}
	}
		
	public function display($echo = true) {
		
		$archives = $this->getArchives();				
		$archives = $this->organizeArchivesInTree($archives);
		$archives = $this->trimArchivesTree($archives);
		
		$output = "";
						
		// Start off wrapper and process the first row
		$output .= '<ul class="PW_Archives years">';
				
		// Loop through each of the years
		foreach($archives->years as $year) {
							
			// display the year
			$output .= '<li class="year">' . $this->listYear($year->post_year, $year->year_count);
			
			// if there are months, display them
			if ($year->months) {

				// open the months list, then looop through each month, within this year
				$output .= '<ul class="months">'; // 
				foreach($year->months as $month) {					
					
					// display the month
					$output .= '<li class="month">' . $this->listMonth($month->post_month, $month->post_year, $month->month_count);
					
					// if there are posts, display them
					if ($month->posts) {

						// open the posts list, the loop through each post
						$output .= '<ul class="posts">';
						foreach($month->posts as $post) {
							$output .= '<li class="post">' . $this->listPost($post) . '</li>';
						}
						$output .= "</ul>"; // end ul.posts
					}
					$output .= '</li>'; // end li.month
				}
				$output .= "</ul>"; // end ul.posts
			}
			$output .= '</li>'; // end li.year
		}		
		$output .= '</ul>'; // end .years.PW_Archives
		
		if ($echo) {
			echo $output;
		} else {
			return $output;
		}	
	}
		
	protected function getArchives() {
		
		global $wpdb, $blog_id;
		
		// set the variables uses in the query
		$blog_id = isset($blog_id) ? $blog_id : ""; // in case the blog is not in multisite mode.
		$posts_table = $wpdb->get_blog_prefix($blog_id) . "posts";

		// join months and years adding columns for counts
		$monthAndYearCountQuery = "SELECT y.post_year AS `post_year`, year_count, m.post_month AS `post_month`, month_count
		FROM (
			SELECT YEAR( post_date ) AS post_year, count( ID ) AS year_count
			FROM $posts_table
			WHERE post_status = 'publish'
			AND post_type = 'post'
			GROUP BY YEAR( post_date )
			ORDER BY post_date DESC
		) AS y
		NATURAL JOIN (
			SELECT YEAR( post_date ) AS post_year, MONTH( post_date ) AS post_month, count( ID ) AS month_count
			FROM $posts_table
			WHERE post_status = 'publish'
			AND post_type = 'post'
			GROUP BY YEAR( post_date ) , MONTH( post_date )
			ORDER BY post_date DESC
		) AS m";

		// all published posts joined with added columns in each row showing month and year counts
		$allPostsWithMonthAndYearCountQuery = "SELECT ID, post_date, post_name, post_title, comment_count, y.post_year AS `post_year`, year_count, m.post_month AS `post_month`, month_count
		FROM (
			SELECT YEAR( post_date ) AS post_year, count( ID ) AS year_count
			FROM $posts_table
			WHERE post_status = 'publish'
			AND post_type = 'post'
			GROUP BY YEAR( post_date )
			ORDER BY post_date DESC
		) AS y
		NATURAL JOIN (
			SELECT YEAR( post_date ) AS post_year, MONTH( post_date ) AS post_month, count( ID ) AS month_count
			FROM $posts_table
			WHERE post_status = 'publish'
			AND post_type = 'post'
			GROUP BY YEAR( post_date ) , MONTH( post_date )
			ORDER BY post_date DESC
		) AS m
	 	NATURAL JOIN (
			SELECT ID, post_date, MONTH( post_date ) AS post_month, YEAR( post_date ) AS post_year, post_name, post_title, comment_count
			FROM $posts_table
			WHERE post_status = 'publish'
			AND post_type = 'post'
			ORDER BY post_date DESC
		) AS p";
		
		if (strcasecmp($this->options['show_posts'], 'YES') === 0) {
			return $wpdb->get_results($allPostsWithMonthAndYearCountQuery);
		} else {
			return $wpdb->get_results($monthAndYearCountQuery);
		}
	}
	
	protected function organizeArchivesInTree($queryResults) {
				
		// organize the query results into an archive tree: years > months > post
		$archives = (object) null;
		$archives->years = array();
		foreach($queryResults as $row) {
			// if no year object for this row exists, populate it
			if (!isset($archives->years[$row->post_year])) {
				$year = (object) null;
				$year->post_year = $row->post_year;
				$year->year_count = $row->year_count;
				$year->months = array();
				$archives->years[$row->post_year] = $year;
			}
			
			// if no month object for this row exists, populate it
			if (!isset($archives->years[$row->post_year]->months[$row->post_month])) {
				$month = (object) null;
				$month->post_month = $row->post_month;
				$month->post_year = $row->post_year;
				$month->month_count = $row->month_count;
				$month->posts = array();
				$archives->years[$row->post_year]->months[$row->post_month] = $month;
			}

			// if posts were included in the query populate the posts object
			if (isset($row->ID)) {
				$row->post_type = 'post';
				$row->post_status = 'publish';
				$row->post_filter = 'sample';
				$archives->years[$row->post_year]->months[$row->post_month]->posts[] = $row;	
			}
		}
		return $archives;		
	}

	protected function trimArchivesTree($archives) {

		// Trim the tree based on what the plugin options of how many years/months/posts should be shown
		if ($this->options['year_count'] != 'ALL') {
			array_splice($archives->years, (int) $this->options['year_count']);
		}
				
		// If not all years should be expanded, trim the ones that won't be
		if ($this->options['month_expand'] != 'ALL')
		{
			$count = 0;
			foreach($archives->years as $year)
			{							
				$count++;
				// If the count is less than the number of years to expand, continue
				if ($count <= (int) $this->options['month_expand']) {				
					continue;
				}
				// remove the months from the year object
				$year->months = array();
			}
			
		}
		
		// This needs to be last in the function, otherwise the ordering will be messed up
		if ($this->options['order'] == "ASC") {
			$archives = $this->reverseArchiveOrder($archives);
		}
		
		return $archives;
	}
	
	protected function reverseArchiveOrder($archives) {
		$archives->years = array_reverse($archives->years, true);
		foreach($archives->years as $year) {			
			$year->months = array_reverse($year->months, true);
			foreach($year->months as $month) {
				$month->posts = array_reverse($month->posts, true);
			}
		}
		return $archives;
	}

	protected function listYear($year, $count) {
		$output = str_replace(
			array("%YEAR%", "%YEAR_URL%", "%POST_COUNT%"),
			array(date($this->options['year_format'], mktime(0,0,0,1,1,$year)), get_year_link($year), $count),
			$this->options['year_template']
		);
		return $output;
	}
	
	protected function listMonth($month, $year, $count) {
		$output = str_replace(
			array("%MONTH%", "%MONTH_URL%", "%POST_COUNT%"),
			array(date($this->options['month_format'], mktime(0,0,0,$month,1,$year)), get_month_link($year, $month), $count),
			$this->options['month_template']
		);
		return $output;
	}
	
	protected function listPost($post) {

		// If the posts variable is set, don't set it again
		if (!$this->posts) {
			 // If the user has explicitely set the option to not use fast permalinks, get posts manually
			if ($this->options['faster_permalinks'] !== 'YES') {
				$this->get_posts();
			} 
			// else, determine if the permalink structure will work with this plugin. If it won't, get posts manually
			else {
				$permalink_structure = get_option('permalink_structure');
				if (!(strpos($permalink_structure, '%category%') === false && strpos($permalink_structure, '%author%') === false)) {
					$this->get_posts();
				}
			}
		}

		// Get the permalink, if bad_permalinks is true, get them the hard way
		//$permalink = $this->bad_permalinks ? get_permalink($this->posts[$post->ID]) : get_permalink($post);
		$permalink = $this->posts ? get_permalink($post->ID) : get_permalink($post);
		
		$output = str_replace(
			array('%POST_TITLE%', '%POST_DATE%', '%POST_URL%', '%COMMENT_COUNT%'),
			//array($title, get_permalink($this->posts[$id]), (int) $comment_count),
			array($post->post_title, date($this->options['post_date_format'], strtotime($post->post_date)), $permalink, (int) $post->comment_count),
			$this->options['post_template']
		);
		return $output;
	}
	
	protected function get_posts() {
		$raw_posts = get_posts(array('numberposts'=>-1));
		foreach($raw_posts as $post) {
			$this->posts[$post->ID] = $post;
		}
	}
	
} // class PW_Archives	

class PW_Archives_Widget extends WP_Widget {
	/**
	 * Widget setup.
	 */
	function PW_Archives_Widget()	{
		/* Widget settings. */
		$widget_ops = array( 'classname' => 'pw_archives', 'description' => 'An easy to use, light-weight, fully-customizable archiving plugin that allows you to display your posts almost any way you like.' );

		/* Widget control settings. */
		$control_ops = array( 'width' => 425, 'height' => 500, 'id_base' => 'pw_archives' );

		/* Create the widget. */
		$this->WP_Widget( 'pw_archives','PW_Arcives', $widget_ops, $control_ops );
	}

	/**
	 * How to display the widget on the screen.
	 */
	function widget( $args, $instance )	{
		extract( $args );
		$title = apply_filters('widget_title', $instance['title'] );
		
		/* Before widget (defined by themes). */
		echo $before_widget;

		/* Display the widget title if one was input (before and after defined by themes). */
		if ( $title )
			echo $before_title . $title . $after_title;
		
		$pwa = new PW_Archives($instance);
		$pwa->display();

		/* After widget (defined by themes). */
		echo $after_widget;
	}

	/**
	 * Update the widget settings.
	 */
	function update( $new_instance, $old_instance )	{
		$new_instance['title'] = strip_tags($new_instance['title']);
		return $new_instance;
	}

	/**
	 * Displays the widget settings controls on the widget panel.
	 * Make use of the get_field_id() and get_field_name() function
	 * when creating your form elements. This handles the confusing stuff.
	 */
	function form( $instance )	{
		/* Set up some default widget settings. */
		
		$options = get_option('PW_Archives_options');
		$options = wp_parse_args($options, PW_Archives::$defaults);
		$instance = wp_parse_args( (array) $instance, $options );
		// $instance = wp_parse_args( (array) $instance, PW_Archives::$defaults);
		
		$h3style = 'style="border-top:1px solid #999; margin:.25em 0; padding:.5em 0;"';
		?>
		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>">Title:</label>
			<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo $instance['title']; ?>" type="text" />
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'order' ); ?>">List archives in
				<select id="<?php echo $this->get_field_id( 'order' ); ?>" name="<?php echo $this->get_field_name( 'order' ); ?>">
					<option value="DESC" <?php selected($instance['order'], 'DESC' ); ?>>descending</option>
					<option value="ASC" <?php selected($instance['order'], 'ASC'); ?>>ascending</option>
				</select>
			order</label>
		</p>
		<h3 <?php echo $h3style;?>>Year Options</h3>
		<p>
			<label for="<?php echo $this->get_field_id( 'year_count' ); ?>">Number of years to display:</label>
			<select id="<?php echo $this->get_field_id( 'year_count' ); ?>" name="<?php echo $this->get_field_name( 'year_count' ); ?>">
				<option value="ALL" <?php selected($instance['year_count'], 'ALL' ); ?>>All</option>
				<?php
					for ($i = 1; $i <= 10; $i++) {
						echo '<option value="' . $i . '" ' . selected($instance['year_count'], $i) . ">$i</option>";
					} 
				?>
			</select>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'year_format' ); ?>">Date format:</label>			
			<input size="2" class="code" id="<?php echo $this->get_field_id( 'year_format' ); ?>" name="<?php echo $this->get_field_name( 'year_format' ); ?>" value="<?php echo $instance['year_format']; ?>" type="text" />
			<small>See the <a href="http://php.net/manual/en/function.date.php">PHP date() documentation</a> for options</small>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'year_template' ); ?>">Template:</label>
			<input class="widefat code" id="<?php echo $this->get_field_id( 'year_template' ); ?>" name="<?php echo $this->get_field_name( 'year_template' ); ?>" value="<?php echo htmlentities($instance['year_template']); ?>" type="text" />
			<small style="color:#666;">Variables: <code>%YEAR%</code>, <code>%YEAR_URL%</code>, and <code>%POST_COUNT%</code></small>
		</p>
		<h3 <?php echo $h3style;?>>Month Options</h3>
		<p>
			<label for="<?php echo $this->get_field_id( 'month_expand' ); ?>">Display months within:</label>
			<select id="<?php echo $this->get_field_id( 'month_expand' ); ?>" name="<?php echo $this->get_field_name( 'month_expand' ); ?>">
				<option value="ALL" <?php selected($instance['month_expand'], 'ALL' ); ?>>All years</option>
				<option value="1" <?php selected($instance['month_expand'], '1' ); ?>>Only the most recent year</option>
				<option value="0" <?php selected($instance['month_expand'], '0' ); ?>>No years</option>
			</select>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'month_format' ); ?>">Date format:</label>
			<input size="2" class="code" id="<?php echo $this->get_field_id( 'month_format' ); ?>" name="<?php echo $this->get_field_name( 'month_format' ); ?>" value="<?php echo $instance['month_format']; ?>" type="text" />
			<small>See the <a href="http://php.net/manual/en/function.date.php">PHP date() documentation</a> for options</small>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'month_template' ); ?>">Template:</label>
			<input class="widefat code" id="<?php echo $this->get_field_id( 'month_template' ); ?>" name="<?php echo $this->get_field_name( 'month_template' ); ?>" value="<?php echo htmlentities($instance['month_template']); ?>" type="text" />
			<small style="color:#666;">Variables: <code>%MONTH%</code>, <code>%MONTH_URL%</code>, and <code>%POST_COUNT%</code></small>
		</p>
		<h3 <?php echo $h3style;?>>Post Options</h3>
		<p>
			<input value="NO" type="hidden" name="<?php echo $this->get_field_name( 'show_posts' ); ?>" />
			<label><input value="YES" class="checkbox" type="checkbox" <?php checked( $instance['show_posts'], "YES" ); ?> name="<?php echo $this->get_field_name( 'show_posts' ); ?>" /> Display posts within visible months</label>
		</p>
		<p>
			<input value="NO" type="hidden" name="<?php echo $this->get_field_name( 'faster_permalinks' ); ?>" />
			<label><input value="YES" class="checkbox" type="checkbox" <?php checked( $instance['faster_permalinks'], "YES" ); ?> name="<?php echo $this->get_field_name( 'faster_permalinks' ); ?>" /> Use PW_Archive's faster permalinks option if possible</label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'post_date_format' ); ?>">Date format:</label>
			<input size="2" class="code" id="<?php echo $this->get_field_id( 'post_date_format' ); ?>" name="<?php echo $this->get_field_name( 'post_date_format' ); ?>" value="<?php echo $instance['post_date_format']; ?>" type="text" />
			<small>See the <a href="http://php.net/manual/en/function.date.php">PHP date() documentation</a> for options</small>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'post_template' ); ?>">Post link template:</label>
			<input class="widefat code" id="<?php echo $this->get_field_id( 'post_template' ); ?>" name="<?php echo $this->get_field_name( 'post_template' ); ?>" value="<?php echo htmlentities($instance['post_template']); ?>" type="text" />
			<small style="color:#666;">Variables: <code>%POST_TITLE%</code>, <code>%POST_DATE%</code>, <code>%POST_URL%</code>, and <code>%COMMENT_COUNT%</code></small>
		</p>
	<?php
	}
}


// Actions/Hooks
add_action('widgets_init', array('PW_Archives', 'register_widget'));
add_action('admin_init', array('PW_Archives', 'initialize_settings'));
add_action('admin_menu', array('PW_Archives', 'add_submenu'));
add_filter('plugin_action_links_'.plugin_basename(__FILE__), array('PW_Archives', 'settings_link'));
add_shortcode( 'PW_Archives', array('PW_Archives', 'shortcode'));